Imports System.ComponentModel

<ToolboxBitmap(GetType(IsRequiredProvider), "required_provider.bmp"), _
 ProvideProperty("IsRequiredMessage", GetType(Control))> _
Public Class IsRequiredProvider
    Inherits System.ComponentModel.Component
    Implements IExtenderProvider

#Region " Component Designer generated code "

    Public Sub New(ByVal Container As System.ComponentModel.IContainer)
        MyClass.New()

        'Required for Windows.Forms Class Composition Designer support
        Container.Add(Me)
    End Sub

    Public Sub New()
        MyBase.New()

        'This call is required by the Component Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'Component overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Component Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Component Designer
    'It can be modified using the Component Designer.
    'Do not modify it using the code editor.
    Friend WithEvents errOutOfRange As System.Windows.Forms.ErrorProvider
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.errOutOfRange = New System.Windows.Forms.ErrorProvider
        '
        'errOutOfRange
        '
        Me.errOutOfRange.BlinkStyle = System.Windows.Forms.ErrorBlinkStyle.NeverBlink

    End Sub

#End Region

    ' The error message to display if a control is missing.
    Private m_MissingMessages As New Hashtable

    Private m_Enabled As Boolean = True
    <Category("Behavior"), _
     DefaultValue(True)> _
    Public Property Enabled() As Boolean
        Get
            Return m_Enabled
        End Get
        Set(ByVal Value As Boolean)
            m_Enabled = Value
        End Set
    End Property

    ' We can extend TextBoxes.
    Public Function CanExtend(ByVal client_control As Object) As Boolean Implements IExtenderProvider.CanExtend
        Return (TypeOf client_control Is TextBox)
    End Function

    ' Return this control's is required message.
    <Category("Validation"), _
     DefaultValue("")> _
    Public Function GetIsRequiredMessage(ByVal client_control As Control) As String
        ' See if we have a message for this control.
        If m_MissingMessages.Contains(client_control) Then
            ' Return the message.
            Return DirectCast(m_MissingMessages(client_control), String)
        Else
            ' We have no message for this control.
            Return Nothing
        End If
    End Function

    ' Set the control's missing message.
    <Category("Validation"), _
     DefaultValue("")> _
    Public Sub SetIsRequiredMessage(ByVal client_control As Control, ByVal missing_message As String)
        ' If this control is now required, remove it.
        If m_MissingMessages.Contains(client_control) Then
            m_MissingMessages.Remove(client_control)
            RemoveHandler client_control.Validating, AddressOf Client_Validating
        End If

        ' If the new message is non-blank, add it.
        If Not (missing_message Is Nothing) Then
            If missing_message.Length > 0 Then
                m_MissingMessages.Add(client_control, missing_message)
                AddHandler client_control.Validating, AddressOf Client_Validating
            End If
        End If
    End Sub

    ' A client is validating. See if it is blank.
    Private Sub Client_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs)
        If Not m_Enabled Then Exit Sub
        ValidateTextBox(DirectCast(sender, TextBox))
    End Sub

    ' Validate this control.
    Private Sub ValidateTextBox(ByVal client_control As TextBox)
        If client_control.Text.Trim.Length > 0 Then
            ' It's not blank.
            Me.errOutOfRange.SetError(client_control, "")
        Else
            ' It's blank.
            Dim missing_message As String
            Me.errOutOfRange.SetError(client_control, DirectCast(m_MissingMessages(client_control), String))
        End If
    End Sub

    ' If some control has an error, display its error message,
    ' set focus to it, and return True. If all controls are
    ' okay, then return False.
    Public Function HasError() As Boolean
        If Not m_Enabled Then Return False

        ' Make sure all controls have been validated.
        For Each text_box As TextBox In m_MissingMessages.Keys
            ValidateTextBox(text_box)
        Next text_box

        ' Find the first control in the tab order with an error.
        Dim first_text_box As TextBox = Nothing
        Dim first_tab_index As Integer = Integer.MaxValue
        For Each text_box As TextBox In m_MissingMessages.Keys
            ' See if the control has an error.
            If errOutOfRange.GetError(text_box).Length > 0 Then
                If text_box.TabIndex < first_tab_index Then
                    first_text_box = text_box
                    first_tab_index = text_box.TabIndex
                End If
            End If
        Next text_box

        ' See if any control has an error.
        If Not (first_text_box Is Nothing) Then
            first_text_box.Focus()
            MessageBox.Show(errOutOfRange.GetError(first_text_box))
            Return True
        End If

        Return False
    End Function
End Class
